import numpy as np
import pylab as pl
import scipy as sp
from scipy import linalg,random

def orthonorm(Y):
    T=Y.copy()
    n,m=T.shape
    if n < m:
        raise ValueError('Dimensions incompatibles')
    Z=np.zeros(shape=(n,m))
    u=T[0:,0]
    u=u/sp.linalg.norm(u)
    Z[0:,0]=u
    for j in range(1,m):
        u=T[0:,j]
        W=np.dot(Z,Z.T)
        u=u-np.dot(W,u)
        u=u/sp.linalg.norm(u)
        Z[0:,j]=u
    return Z

def householder(a):
    x=a.copy()
    normx=sp.linalg.norm(x)
    rho=-np.sign(x[0])
    u1=x[0]-rho*normx
    u=x/u1
    u[0]=1
    u=u/sp.linalg.norm(u)
    return u   

def qr_householder(A):
    # renvoie  dans l'ordre Q,R tq A=QR
    n,m=A.shape
    s=min(n,m)
    R=A.copy()
    H=np.eye(n)
    for k in range(s-1):
        u=householder(R[k:,k])
        u.shape=(n-k,1)
        tH1=np.eye(n-k)-2*np.dot(u,u.T)
        H1=np.eye(n)
        H1[k:,k:]=tH1
        B=np.zeros(shape=(n,m))
        for j in range(m):
            a=R[0:,j]
            a.shape=(n,1)
            c=np.dot(H1,a)
            c.shape=(n,)
            B[0:,j]=c
        #print(B)
        R=B.copy()
        H=np.dot(H1,H)
    return H.T,R

def puissance(A,N):
    A0=A.copy()
    n=len(A[0])
    x0=2*np.random.rand(n,1)
    x0 -= np.ones(shape=(n,1))
    x=x0.copy()
    for k in range(N):
        y=np.dot(A0,x)
        nu=np.dot(x.T,y)
        x=y/sp.linalg.norm(y)
    return x,nu    

def puissance_m(A,N):
    A0=A.copy()
    n=len(A[0])
    x0=2*np.random.rand(n,n)
    x0 -= np.ones(shape=(n,n))
    x=x0.copy()
    for k in range(N):
        y=np.dot(A0,x)
        nu=np.dot(x.T,y)
        x=orthonorm(y) #y/sp.linalg.norm(y)
    return x,nu    


def QRval(A,it):
    # retourne dans l'ordre Q,R,taberr
    A0=A.copy()
    n,m=A.shape
    if n != m:
        raise ValueError('Dimensions incompatibles')
    Q=np.eye(n)
    taberr=np.zeros(it)
    for k in range(it):
        Qk,Rk=qr_householder(A0)
        A1=np.dot(Rk,Qk)
        A0=A1.copy()
        Q=np.dot(Q,Qk)
        taberr[k]=sp.linalg.norm(A1-np.triu(A1))
    return Q,A1,taberr    # --> Q,R,taberr

def QRval_eps(A,eps):
    # retourne dans l'ordre Q,R,taberr
    A0=A.copy()
    n,m=A.shape
    if n != m:
        raise ValueError('Dimensions incompatibles')
    Q=np.eye(n)
    itmax=100
    taberr=np.zeros(itmax+1)
    it=0
    test=sp.linalg.norm(A0-np.triu(A0))
    taberr[0]=test
    while test > eps and it < itmax:
        it += 1
        Qk,Rk=qr_householder(A0)
        A1=np.dot(Rk,Qk)
        A0=A1.copy()
        Q=np.dot(Q,Qk)
        test=sp.linalg.norm(A1-np.triu(A1))
        taberr[it]=test
        #it += 1
    return Q,A1,taberr,it    # --> Q,R,taberr,it

        


